{ open Parser }

let letter = ['a'-'z' 'A'-'Z']
let digit  = ['0'-'9']
let idchar = ['a'-'z' 'A'-'Z' '0'-'9' '_' '\'']

rule token = parse
    [' ' '\t' '\n']       { token lexbuf }      (* whitespace *)
  | "--" [^ '\n']* '\n'   { token lexbuf }      (* comment *)
  | '('                   { LPAREN }
  | ')'                   { RPAREN }
  | "[["                  { LDOUBLE }
  | "]]"                  { RDOUBLE }
  | '['                   { LSQUARE }
  | ']'                   { RSQUARE }
  | '{'                   { LCURLY }
  | '}'                   { RCURLY }
  | ';'                   { SEMI }
  | "<-"                  { ASSIGN }
  | "<<"                  { LSHIFT }
  | ">>"                  { RSHIFT }
  | '*'                   { STAR }
  | '/'                   { SLASH }
  | '%'                   { MOD }
  | '+'                   { PLUS }
  | '-'                   { MINUS }
  | '|'                   { VBAR }
  | '^'                   { CARET }
  | '~'                   { TILDE }
  | '&'                   { AMP }
  | '='                   { EQ }
  | "!="                  { NE }
  | "<="                  { LE }
  | ">="                  { GT }
  | '<'                   { LT }
  | '>'                   { GT }
  | ':'                   { COLON }
  | ','                   { COMMA }
  | "::"                  { CONVERT }
  | '.'                   { DOT }
  | ".."                  { TWODOTS }
  | "..."                 { THREEDOTS }
  | "//"                  { MAP }
  | "/."                  { REDUCE }
  | "/:"                  { SCANL }
  | ":/"                  { SCANR }
  | "and"                 { AND }
  | "break"               { BREAK }
  | "continue"            { CONTINUE }
  | "else"                { ELSE }
  | "if"                  { IF }
  | "for"                 { FOR }
  | "func"                { FUNC }
  | "global"              { GLOBAL }
  | "kernel"              { KERNEL }
  | "local"               { LOCAL }
  | "not"                 { NOT }
  | "or"                  { OR }
  | "pass"                { PASS }
  | "return"              { RETURN }
  | "shared"              { SHARED }
  | "sync"                { SYNC }
  | "then"                { THEN }
  | "type"                { TYPE }
  | "while"               { WHILE }
  (* Basic types and constant literals *)
  | "Bool"                { BOOL }
  | "Void"                { VOID }
  | "Int32"               { INT32 }
  | "Int64"               { INT64 }
  | "Uint32"              { UINT32 }
  | "Uint64"              { UINT64 }
  | "Float32"             { FLOAT32 }
  | "Float64"             { FLOAT64 }
  | "TRUE"                { TRUE }
  | "FALSE"               { FALSE }
  | eof                   { EOF }
  | digit+ as inum        { INTLITERAL(int_of_string inum) } (* pos. integers *)
  | digit+ '.' digit+ as fnum { FLOATLITERAL(float_of_string fnum) }
  | letter idchar* as id  { ID(id) }
  | _ as bogus            { raise (Failure("Bad token: " ^ Char.escaped bogus)) }
